"use strict";
/******************************************************************
 MIT License http://www.opensource.org/licenses/mit-license.php
 Author Mora <qiuzhongleiabc@126.com> (https://github.com/qiu8310)
*******************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
var parseAttrs_1 = require("../parseAttrs");
describe('single line', function () {
    test('should parse basic single line', function () {
        expect(parseAttrs_1.parseAttrs("class {\n      properties = {foo: Number, bar: String}\n    }\n    ")).toEqual([
            { name: 'foo', type: { name: 'number' } },
            { name: 'bar', type: { name: 'string' } },
        ]);
    });
    test('should parse complex single line', function () {
        expect(parseAttrs_1.parseAttrs("  properties: {foo: Number, bar: {type: String, others: {type: Number}}}")).toEqual([
            { name: 'foo', type: { name: 'number' } },
            { name: 'bar', type: { name: 'string' } },
        ]);
    });
    test('should parse single line with doc', function () {
        expect(parseAttrs_1.parseAttrs("  properties: {/** \u8FD9\u662F foo \u7684\u63CF\u8FF0 */foo: Number}")).toEqual([
            { name: 'foo', type: { name: 'number' }, desc: ['这是 foo 的描述'] },
        ]);
    });
});
describe('multiple lines', function () {
    test('should parse basic multiple lines', function () {
        expect(parseAttrs_1.parseAttrs("\n    class {\n      properties = {\n        foo: Number,\n        bar: String\n      }\n    }\n    ")).toEqual([
            { name: 'foo', type: { name: 'number' } },
            { name: 'bar', type: { name: 'string' } },
        ]);
    });
    test('should parse complex multiple lines', function () {
        expect(parseAttrs_1.parseAttrs("\n      properties = {\n        foo: Number,\n        bar: {\n          type: String,\n          value: '123',\n          others: {\n            type: Number\n          },\n          fn: () => {\n            return String\n          }\n        }\n      }\n    ")).toEqual([
            { name: 'foo', type: { name: 'number' } },
            { name: 'bar', type: { name: 'string' }, defaultValue: '123' },
        ]);
    });
    test('should parse multiple lines with desc', function () {
        expect(parseAttrs_1.parseAttrs("\n      properties = {\n        /** \u8FD9\u662F foo \u7684\u63CF\u8FF0 */\n        foo: Number,\n        /**\n         * \u8FD9\u662F bar \u7684\u63CF\u8FF0\n         * \u7B2C\u4E00\u884C\n         *\n         * \u8FD9\u662F\u7B2C\u4E8C\u884C\n         *\n         * @required\n         * @type string\n         * @default 12\n         *\n         * @ \u540E\u9762\u7684\u5168\u4E0D\u8981\u4E86\n         */\n        bar: Number\n      }\n    ")).toEqual([
            { name: 'foo', type: { name: 'number' }, desc: ['这是 foo 的描述'] },
            {
                name: 'bar',
                type: { name: 'string' },
                desc: ['这是 bar 的描述第一行', '这是第二行'],
                required: true,
                defaultValue: 12,
            },
        ]);
    });
});
